Package: GBScleanR
Type: Package
Title: Error correction tool for noisy genotyping by sequencing (GBS)
        data
Version: 2.7.2
Date: 2026-07-13
Authors@R: person("Tomoyuki", "Furuta", 
           email = "f.tomoyuki@okayama-u.ac.jp",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0869-6626"))
Description: GBScleanR is a package for quality check, filtering, and error 
   correction of genotype data derived from next generation sequcener (NGS)
   based genotyping platforms. GBScleanR takes Variant Call Format (VCF) file as
   input. The main function of this package is `estGeno()` which estimates the 
   true genotypes of samples from given read counts for genotype markers using a
   hidden Markov model with incorporating uneven observation ratio of allelic 
   reads. This implementation gives robust genotype estimation even in noisy 
   genotype data usually observed in Genotyping-By-Sequnencing (GBS) and similar
   methods, e.g. RADseq. The current implementation accepts genotype data of a 
   diploid population at any generation of multi-parental cross, e.g. biparental
   F2 from inbred parents, biparental F2 from outbred parents, and 8-way 
   recombinant inbred lines (8-way RILs) which can be refered to as MAGIC 
   population.
License: GPL-3 + file LICENSE
Encoding: UTF-8
LinkingTo: Rcpp, RcppParallel
SystemRequirements: GNU make, C++11
Depends: SeqArray
Imports: stats, utils, methods, ggplot2, tidyr, expm, Rcpp,
        RcppParallel, gdsfmt
Suggests: BiocStyle, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
biocViews: GeneticVariability, SNP, Genetics, HiddenMarkovModel,
        Sequencing, QualityControl
BugReports: https://github.com/tomoyukif/GBScleanR/issues
URL: https://github.com/tomoyukif/GBScleanR
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/GBScleanR
git_branch: devel
git_last_commit: 09dd0ee
git_last_commit_date: 2026-07-13
Repository: Bioconductor 3.24
Date/Publication: 2026-07-26
NeedsCompilation: yes
Packaged: 2026-07-26 22:22:23 UTC; biocbuild
Author: Tomoyuki Furuta [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0869-6626>)
Maintainer: Tomoyuki Furuta <f.tomoyuki@okayama-u.ac.jp>
